# coding: utf-8

from __future__ import print_function

"""
conversion helpers for yaml utility to/from HTML

html/xml to yaml inspired by
http://programmers.stackexchange.com/questions/157395/yaml-translating-free-flowing-text-from-xml

"""

import sys


class HTML2YAML(object):
    def __init__(self, args=None):
        self.flatten = not getattr(args, 'no_flatten', False)
        self.strip = getattr(args, 'strip', False)
        self.no_body = getattr(args, 'no_body', False)

    def __call__(self, html):
        d = self.html_to_data(html)
        if self.no_body:
            d = d['html']['body']
        return self.data_to_yaml(d)

    def data_to_yaml(self, d):
        import ruamel.yaml
        return ruamel.yaml.dump(
            d,
            Dumper=ruamel.yaml.RoundTripDumper,
            allow_unicode=True,
        )

    def html_to_data(self, html):
        try:
            import bs4
        except ImportError:
            print("For HTML conversion you need to install BeautifulSoup")
            print("e.g. using (pip install beautifulsoup4)")
            sys.exit(1)

        soup = bs4.BeautifulSoup(html)
        data = self._convert_node(soup)
        return data

    def _convert_node(self, node, depth=0):
        try:
            import bs4
        except ImportError:
            print("For HTML conversion you need to install BeautifulSoup")
            print("e.g. using (pip install beautifulsoup4)")
            sys.exit(1)
        from ruamel.yaml.comments import CommentedMap
        from ruamel.yaml.scalarstring import PreservedScalarString
        ret_val = []
        if node.attrs:
            ret_val.append({'.attribute': node.attrs})
        for data in node.contents:
            if isinstance(data, bs4.Tag):
                kv = CommentedMap()
                #print data.name, data.attrs
                # convert the intenals of the tag
                kv[data.name] = self._convert_node(data, depth+1)
                ret_val.append(kv)
            elif isinstance(data, bs4.NavigableString):
                s, nl = self._strip(data)
                if not s:
                    continue
                if nl:
                    ret_val.append(PreservedScalarString(s))
                    continue
                ret_val.append(s)
            else:
                print('unknow type', type(data))
        if self.flatten and len(ret_val) == 1:
            return ret_val[0]
        return ret_val

    def _strip(self, data):
        import textwrap
        # multiline strings might be nicely formatted so don't
        # use .strip() immediately
        if self.strip:
            s = data.strip()
        else:
            s = data.rstrip()
        if not s:
            return None, False
        first_nl_pos = s.find(u'\n')
        if first_nl_pos < 0:
            return s, False
        if not s[:first_nl_pos].strip():  # i.e. space until first newline
            if u'\n' not in s[first_nl_pos+1:]:
                print(repr(data), repr(s))
                # single line of text preceded and followed by nl
                return s.strip(), False
            # use data here, removing the final newline would get your |- as marker
            s = textwrap.dedent(data[first_nl_pos+1:])
        return s, True


class YAML2HTML(object):
    def __init__(self, args=None):
        pass

    def __call__(self, yaml):
        d = self.yaml_to_data(yaml)
        return self.data_to_html(d)

    def data_to_html(self, d):
        if isinstance(d, dict):
            pass

    def yaml_to_data(self, yaml):
        import ruamel.yaml
        return ruamel.yaml.load(yaml)
        return data





from __future__ import print_function

import io
import sys
import subprocess

from roundtrip import dedent

from test_util import check_output, call_util


def to_html(s, file_name, mp, td, options=None):
    cmd = ['yaml', 'htmltable', file_name]
    if options:
        cmd.extend(options)
    return call_util(s, file_name, cmd, mp, td)


class TestUtilToHTML:
    def test_html_01(self, tmpdir, monkeypatch):
        file_name = "html.yaml"
        res = to_html(u"""
        - 1
        - 2
        """, file_name, mp=monkeypatch, td=tmpdir, options=["--level"])
        assert res == "levels: 1\n"

    def test_html_02(self, tmpdir, monkeypatch):
        file_name = "html.yaml"
        res = to_html(u"""
        abc:
        - 1
        - 2
        ghi:
        - 3
        - 4
        """, file_name, mp=monkeypatch, td=tmpdir, options=["--level"])
        assert res == "levels: 2\n".format(file_name)

    def test_html_03(self, tmpdir, monkeypatch):
        file_name = "html.yaml"
        res = to_html(u"""
        -
          - 1
          - 2
        -
          - 3
          - 4
        """, file_name, mp=monkeypatch, td=tmpdir, options=["--level"])
        assert res == "levels: 2\n"

    def test_html_03(self, tmpdir, monkeypatch):
        file_name = "html.yaml"
        res = to_html(u"""
        -
          abc:
          - 1
          - 2
          def:
          - 3
          - 4
        """, file_name, mp=monkeypatch, td=tmpdir, options=["--level"])
        assert res == "levels: 3\n"

    def test_html_04(self, tmpdir, monkeypatch):
        file_name = "html.yaml"
        res = to_html(u"""
        title:
        - fruit
        - legume
        local:
        - apple
        - sprouts
        import:
        - orange
        - broccoli
        """, file_name, mp=monkeypatch, td=tmpdir)
        assert res == dedent("""\
        <HTML>
        <HEAD>
        </HEAD>
        <BODY>
        <TABLE>
          <TR>
            <TD>title</TD>
            <TD>fruit</TD>
            <TD>legume</TD>
          </TR>
          <TR>
            <TD>local</TD>
            <TD>apple</TD>
            <TD>sprouts</TD>
          </TR>
          <TR>
            <TD>import</TD>
            <TD>orange</TD>
            <TD>broccoli</TD>
          </TR>
        <TABLE>
        </BODY>
        </HTML>

        """)




from ruamel.yaml.convert.html import HTML2YAML
from textwrap import dedent

class Bin:
    pass

class TestH2Y:
    sample1 = dedent("""\
        text:
        - |-
          This is an example text, spanning multiple lines, and it has embedded elements
          like
        - a:
          - .attribute:
              p: value
          - this
        - and
        - b: this
        - '.  There is also a list:'
        - quote:
          - text:
            - |-
              The text of the quote, spanning multiple lines, and it has
                      embedded elements like
            - c:
              - .attribute:
                  p: value
              - this
            - and
            - b: this
          - author: The Author of this quote
        - Text continues here.
        """)


    def test_00(self):
        b = Bin()
        b.strip = True
        h2y = HTML2YAML(b)
        d = h2y.html_to_data(dedent("""\
        <text>
        This is an example text, spanning multiple lines, and it has embedded elements
        like <a p="value">this</a> and <b>this</b>.  There is also a list:
             <quote>
                <text>The text of the quote, spanning multiple lines, and it has
                embedded elements like <c p="value">this</c> and <b>this</b></text>
                <author>The Author of this quote</author>
            </quote>
        Text continues here.
        </text>
        """))
        if 'html' in d:
            d = d['html']['body']
        res = h2y.data_to_yaml(d)
        assert res == self.sample1


    def XXtest_01(self):
        y2h = YAML2HTML(b)
        d = y2h.yaml_to_data(self.sample1)



# coding: utf-8

"""
testing of anchors and the aliases referring to them
"""

import pytest
from textwrap import dedent

import ruamel.yaml
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump

def load(s):
    return round_trip_load(dedent(s))

def compare(data, s):
    assert round_trip_dump(data) == dedent(s)


class TestAnchorsAliases:
    def test_anchor_id_renumber(self):
        from ruamel.yaml.serializer import Serializer
        assert Serializer.ANCHOR_TEMPLATE == 'id%03d'
        data = load("""
        a: &id002
          b: 1
          c: 2
        d: *id002
        """)
        compare(data, """
        a: &id001
          b: 1
          c: 2
        d: *id001
        """)

    def test_template_matcher(self):
        """test if id matches the anchor template"""
        from ruamel.yaml.serializer import templated_id
        assert templated_id(u'id001')
        assert templated_id(u'id999')
        assert templated_id(u'id1000')
        assert templated_id(u'id0001')
        assert templated_id(u'id0000')
        assert not templated_id(u'id02')
        assert not templated_id(u'id000')
        assert not templated_id(u'x000')

    #def test_re_matcher(self):
    #    import re
    #    assert re.compile(u'id(?!000)\\d{3,}').match('id001')
    #    assert not re.compile(u'id(?!000\\d*)\\d{3,}').match('id000')
    #    assert re.compile(u'id(?!000$)\\d{3,}').match('id0001')

    def test_anchor_assigned(self):
        from ruamel.yaml.comments import CommentedMap
        data = load("""
        a: &id002
          b: 1
          c: 2
        d: *id002
        e: &etemplate
          b: 1
          c: 2
        f: *etemplate
        """)
        d = data['d']
        assert isinstance(d, CommentedMap)
        assert d.yaml_anchor() is None  # got dropped as it matches pattern
        e = data['e']
        assert isinstance(e, CommentedMap)
        assert e.yaml_anchor().value == 'etemplate'
        assert e.yaml_anchor().always_dump is False

    #@pytest.mark.xfail
    def test_anchor_id_retained(self):
        data = load("""
        a: &id002
          b: 1
          c: 2
        d: *id002
        e: &etemplate
          b: 1
          c: 2
        f: *etemplate
        """)
        compare(data, """
        a: &id001
          b: 1
          c: 2
        d: *id001
        e: &etemplate
          b: 1
          c: 2
        f: *etemplate
        """)

    def test_alias_before_anchor(self):
        from ruamel.yaml.composer import ComposerError
        with pytest.raises(ComposerError):
            data = load("""
            d: *id002
            a: &id002
              b: 1
              c: 2
            """)

    def test_anchor_on_sequence(self):
        # as reported by Bjorn Stabell
        # https://bitbucket.org/ruamel/yaml/issue/7/anchor-names-not-preserved
        from ruamel.yaml.comments import CommentedSeq
        data = load("""
        nut1: &alice
         - 1
         - 2
        nut2: &blake
         - some data
         - *alice
        nut3:
         - *blake
         - *alice
        """)
        l = data['nut1']
        assert isinstance(l, CommentedSeq)
        assert l.yaml_anchor() is not None
        assert l.yaml_anchor().value == 'alice'


    merge_yaml = dedent("""
        - &CENTER {x: 1, y: 2}
        - &LEFT {x: 0, y: 2}
        - &BIG {r: 10}
        - &SMALL {r: 1}
        # All the following maps are equal:
        # Explicit keys
        - x: 1
          y: 2
          r: 10
          label: center/big
        # Merge one map
        - <<: *CENTER
          r: 10
          label: center/big
        # Merge multiple maps
        - <<: [*CENTER, *BIG]
          label: center/big
        # Override
        - <<: [*BIG, *LEFT, *SMALL]
          x: 1
          label: center/big
        """)

    def test_merge_00(self):
        data = load(self.merge_yaml)
        d = data[4]
        ok = True
        for k in d:
            for o in [5, 6, 7]:
                if d.get(k) != data[o].get(k):
                    ok = False
                    print('key', k, d.get(k), data[o].get(k))
        assert ok

    def test_merge_accessible(self):
        from ruamel.yaml.comments import CommentedMap, merge_attrib
        data = load("""
        k: &level_2 { a: 1, b2 }
        l: &level_1 { a: 10, c: 3 }
        m:
          << : *level_1
          c: 30
          d: 40
        """)
        d = data['m']
        assert isinstance(d, CommentedMap)
        assert hasattr(d, merge_attrib)

    def test_merge_01(self):
        data = load(self.merge_yaml)
        compare(data, self.merge_yaml)

    def test_add_anchor(self):
        from ruamel.yaml.comments import CommentedMap
        data = CommentedMap()
        data_a = CommentedMap()
        data['a'] = data_a
        data_a['c'] = 3
        data['b'] = 2
        data.yaml_set_anchor('klm', always_dump=True)
        data['a'].yaml_set_anchor('xyz', always_dump=True)
        compare(data, """
        &klm
        a: &xyz
          c: 3
        b: 2
        """)



from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals


from textwrap import dedent

import pytest

import ruamel.yaml


def rt(s):
    return ruamel.yaml.dump(
        ruamel.yaml.load(s, Loader=ruamel.yaml.RoundTripLoader),
        Dumper=ruamel.yaml.RoundTripDumper,
    ).strip() + '\n'


def test_roundtrip_inline_list():
    s = 'a: [a, b, c]\n'
    output = rt(s)
    assert s == output

def test_roundtrip_mapping_of_inline_lists():
    s = dedent("""\
    a: [a, b, c]
    j: [k, l, m]
    """)
    output = rt(s)
    assert s == output

def test_roundtrip_mapping_of_inline_lists_comments():
    s = dedent("""\
    # comment A
    a: [a, b, c]
    # comment B
    j: [k, l, m]
    """)
    output = rt(s)
    assert s == output

def test_roundtrip_mapping_of_inline_sequence_eol_comments():
    s = dedent("""\
    # comment A
    a: [a, b, c]  # comment B
    j: [k, l, m]  # comment C
    """)
    output = rt(s)
    assert s == output


# first test by explicitly setting flow style
def test_added_inline_list():
    s1 = dedent("""
    a:
    - b
    - c
    - d
    """)
    s = 'a: [b, c, d]\n'
    data = ruamel.yaml.load(s1, Loader=ruamel.yaml.RoundTripLoader)
    val = data['a']
    val.fa.set_flow_style()
    # print(type(val), '_yaml_format' in dir(val))
    output = ruamel.yaml.dump(data, Dumper=ruamel.yaml.RoundTripDumper)
    assert s == output

############# flow mappings

def test_roundtrip_flow_mapping():
    s = dedent("""\
    - {a: 1, b: hallo}
    - {j: fka, k: 42}
    """)
    data = ruamel.yaml.load(s, Loader=ruamel.yaml.RoundTripLoader)
    output = ruamel.yaml.dump(data, Dumper=ruamel.yaml.RoundTripDumper)
    assert s == output

def test_roundtrip_sequence_of_inline_mappings_eol_comments():
    s = dedent("""\
    # comment A
    - {a: 1, b: hallo}  # comment B
    - {j: fka, k: 42}  # comment C
    """)
    output = rt(s)
    assert s == output



############# indentation

@pytest.mark.xfail
def test_roundtrip_four_space_indents():
    s = (
        'a:\n'
        '-   foo\n'
        '-   bar\n'
    )
    output = rt(s)
    assert s == output



# coding: utf-8

"""
this is the source for the yaml utility
"""

from __future__ import print_function
from __future__ import absolute_import


import sys
import os
import io
from textwrap import dedent

from ruamel.std.argparse import ProgramBase, option, sub_parser, version, \
    CountAction, SmartFormatter
# from ruamel.appconfig import AppConfig
from . import __version__

import ruamel.yaml
from ruamel.yaml.compat import ordereddict, DBG_TOKEN, DBG_EVENT, DBG_NODE
from ruamel.yaml.configobjwalker import configobj_walker


def yaml_to_html2(code):
    buf = io.StringIO()
    buf.write(u'<HTML>\n')
    buf.write(u'<HEAD>\n')
    buf.write(u'</HEAD>\n')
    buf.write(u'<BODY>\n')
    buf.write(u'<TABLE>\n')
    if isinstance(code, dict):
        for k in code:
            buf.write(u'  <TR>\n')
            for x in [k] + code[k]:
                buf.write(u'    <TD>{0}</TD>\n'.format(x))
            buf.write(u'  </TR>\n')
    buf.write(u'<TABLE>\n')
    buf.write(u'</BODY>\n')
    buf.write(u'</HTML>\n')
    return buf.getvalue()


def yaml_to_html(code, level):
    if level == 2:
        return yaml_to_html2(code)
    elif level == 3:
        return yaml_to_html3(code)
    raise NotImplementedError


class YAML:
    def __init__(self, args, config):
        self._args = args
        self._config = config

    def from_ini(self):
        try:
            from configobj import ConfigObj
        except ImportError:
            print("to convert from .ini you need to install configobj:")
            print("  pip install configobj:")
            sys.exit(1)
        errors = 0
        doc = []
        cfg = ConfigObj(open(self._args.file))
        if self._args.test:
            print(ruamel.yaml.dump(cfg))
            return
        for line in configobj_walker(cfg):
            doc.append(line)
        joined = '\n'.join(doc)
        rto = self.round_trip_single(joined)
        if self._args.basename:
            out_fn = os.path.splitext(self._args.file)[0] + '.yaml'
            if self._args.verbose > 0:
                print('writing', out_fn)
            with open(out_fn, 'w') as fp:
                print(rto, end='', file=fp)  # already has eol at eof
        else:
            print(rto, end='')  # already has eol at eof
        # print()
        # if rto != joined:
        #     self.diff(joined, rto, "test.ini")
        return 1 if errors else 0

    def test(self):
        self._args.event = self._args.node = True
        dbg = 0
        if self._args.event:
            dbg |= DBG_EVENT
        if self._args.node:
            dbg |= DBG_NODE
        os.environ['YAMLDEBUG'] = str(dbg)
        if False:
            x = ruamel.yaml.comment.Comment()
            print(sys.getsizeof(x))
            return

        def print_input(input):
            print(input, end='')
            print('-' * 15)

        def print_tokens(input):
            print('Tokens (from scanner) ' + '#' * 50)
            tokens = ruamel.yaml.scan(input, ruamel.yaml.RoundTripLoader)
            for idx, token in enumerate(tokens):
                # print(token.start_mark)
                # print(token.end_mark)
                print("{0:2} {1}".format(idx, token))

        def rt_events(input):
            dumper = ruamel.yaml.RoundTripDumper
            events = ruamel.yaml.parse(input, ruamel.yaml.RoundTripLoader)
            print(ruamel.yaml.emit(events, indent=False, Dumper=dumper))

        def rt_nodes(input):
            dumper = ruamel.yaml.RoundTripDumper
            nodes = ruamel.yaml.compose(input, ruamel.yaml.RoundTripLoader)
            print(ruamel.yaml.serialize(nodes, indent=False, Dumper=dumper))

        def print_events(input):
            print('Events (from parser) ' + '#' * 50)
            events = ruamel.yaml.parse(input, ruamel.yaml.RoundTripLoader)
            for idx, event in enumerate(events):
                print("{0:2} {1}".format(idx, event))

        def print_nodes(input):
            print('Nodes (from composer) ' + '#' * 50)
            x = ruamel.yaml.compose(input, ruamel.yaml.RoundTripLoader)
            x.dump()  # dump the node

        def scan_file(file_name):
            inp = open(file_name).read()
            print('---------\n', file_name)
            print('---', repr(self.first_non_empty_line(inp)))
            print('<<<', repr(self.last_non_empty_line(inp)))

        if False:
            for x in self._args.file:
                scan_file(x)
            return

        if True:
            import pickle
            lines = 0
            for x in self._args.file:
                print(x, end=' ')
                if x.endswith('.yaml'):
                    data = ruamel.yaml.load(open(x))
                    print(len(data), end=' ')
                    lines += len(data)
                    out_name = x.replace('.yaml', '.pickle')
                    with open(out_name, 'w') as fp:
                        pickle.dump(data, fp)
                elif x.endswith('.pickle'):
                    with open(x) as fp:
                        data = pickle.load(fp)
                    print(len(data), end=' ')
                    lines += len(data)
                print()
            print('lines', lines)
            return

        input = dedent("""
        application: web2py
        version: 1
        runtime: python27
        api_version: 1
        threadsafe: false

        default_expiration: "24h"

        handlers:
        - url: /(?P<a>.+?)/static/(?P<b>.+)
          static_files: 'applications/\\1/static/\\2'
          upload: applications/(.+?)/static/(.+)
          secure: optional
        """)

        input = dedent("""\
        a:
            b: foo
            c: bar
        """)

        print_input(input)
        print_tokens(input)
        print_events(input)
        # rt_events(input)
        print_nodes(input)
        # rt_nodes(input)

        data = ruamel.yaml.load(input, ruamel.yaml.RoundTripLoader)
        print('data', data)
        if False:
            data['american'][0] = 'Fijenoord'
            l = data['american']
        l = data
        if True:
            # print type(l), '\n', dir(l)
            comment = getattr(l, '_yaml_comment', None)
            print('comment_1', comment)
        dumper = ruamel.yaml.RoundTripDumper
        print('>>>>>>>>>>')
        # print(ruamel.yaml.dump(data, default_flow_style=False,
        #    Dumper=dumper), '===========')
        print("{0}=========".format(ruamel.yaml.dump(
            data,
            indent=4,
            Dumper=dumper)))
        comment = getattr(l, '_yaml_comment', None)
        print('comment_2', comment)

        # test end

    def from_json(self):
        # use roundtrip to preserve order
        errors = 0
        docs = []
        for file_name in self._args.file:
            if file_name == '-':
                inp = sys.stdin.read()
            else:
                inp = open(file_name).read()
            loader = ruamel.yaml.Loader # RoundTripLoader
            docs.append(ruamel.yaml.load(inp, loader))
        #if self._args.literal:
        #    from ruamel.yaml.convert.literal import walk_tree
        #    for doc in docs:
        #        walk_tree(doc)
        dumper = ruamel.yaml.RoundTripDumper
        print(ruamel.yaml.dump_all(
            docs, Dumper=dumper,
            default_flow_style=self._args.flow))
        return 1 if errors else 0

    def to_htmltable(self):
        def vals(x):
            if isinstance(x, list):
                return x
            if isinstance(x, (dict, ordereddict)):
                return x.values()
            return []

        def seek_levels(x, count=0):
            my_level = count
            sub_level = 0
            for v in vals(x):
                if v is None:
                    continue
                sub_level = max(sub_level, seek_levels(v, my_level+1))
            return max(my_level, sub_level)

        inp = open(self._args.file).read()
        loader = ruamel.yaml.RoundTripLoader
        code = ruamel.yaml.load(inp, loader)
        # assert isinstance(code, [ruamel.yaml.comments.CommentedMap])
        assert isinstance(code, (dict, list))
        levels = seek_levels(code)
        if self._args.level:
            print("levels:", levels)
            return
        print(yaml_to_html(code, levels))

    def from_html(self):
        from .convert.html import HTML2YAML
        h2y = HTML2YAML(self._args)
        with open(self._args.file) as fp:
            print(h2y(fp.read()))

    def from_csv(self):
        from .convert.from_csv import CSV2YAML
        c2y = CSV2YAML(self._args)
        c2y(self._args.file)

    def round_trip(self):
        errors = 0
        warnings = 0
        for file_name in self._args.file:
            inp = open(file_name).read()
            e, w, stabilize, outp = self.round_trip_input(inp)
            if w == 0:
                if self._args.verbose > 0:
                    print(u"{0}: ok".format(file_name))
                continue
            if self._args.save:
                backup_file_name = file_name + '.orig'
                if not os.path.exists(backup_file_name):
                    os.rename(file_name, backup_file_name)
                with open(file_name, 'w') as ofp:
                    ofp.write(outp)
            if not self._args.save or self._args.verbose > 0:
                print("{0}:\n     {1}".format(file_name, u', '.join(stabilize)))
                self.diff(inp, outp, file_name)
            errors += e
            warnings += w
        if errors > 0:
            return 2
        if warnings > 0:
            return 1
        return 0

    def round_trip_input(self, inp):
        errors = 0
        warnings = 0
        stabilize = []
        outp = self.round_trip_single(inp)
        if inp == outp:
            return errors, warnings, stabilize, outp
        warnings += 1
        if inp.split() != outp.split():
            errors += 1
            stabilize.append(u"drops info on round trip")
        else:
            if self.round_trip_single(outp) == outp:
                stabilize.append(u"stabilizes on second round trip")
            else:
                errors += 1
        ncoutp = self.round_trip_single(inp, drop_comment=True)
        if self.round_trip_single(ncoutp, drop_comment=True) == ncoutp:
            stabilize.append(u"ok without comments")
        return errors, warnings, stabilize, outp

    def round_trip_single(self, inp, drop_comment=False):
        explicit_start=self.first_non_empty_line(inp) == '---'
        explicit_end=self.last_non_empty_line(inp) == '...'
        indent = self._args.indent
        loader = ruamel.yaml.SafeLoader if drop_comment else \
            ruamel.yaml.RoundTripLoader
        code = ruamel.yaml.load(inp, loader)
        dumper = ruamel.yaml.SafeDumper if drop_comment else \
            ruamel.yaml.RoundTripDumper
        return ruamel.yaml.dump(
            code,
            Dumper=dumper,
            indent=indent,
            explicit_start=explicit_start,
            explicit_end=explicit_end,
        )

    def first_non_empty_line(self, txt):
        """return the first non-empty line of a block of text (stripped)
        do not split or strip the complete txt
        """
        pos = txt.find('\n')
        prev_pos = 0
        while pos >= 0:
            segment = txt[prev_pos:pos].strip()
            if segment:
                break
            # print (pos, repr(segment))
            prev_pos = pos
            pos = txt.find('\n', pos+1)
        return segment

    def last_non_empty_line(self, txt):
        """return the last non-empty line of a block of text (stripped)
        do not split or strip the complete txt
        """
        pos = txt.rfind('\n')
        prev_pos = len(txt)
        maxloop = 10
        while pos >= 0:
            segment = txt[pos:prev_pos].strip()
            if segment:
                break
            # print (pos, repr(segment))
            prev_pos = pos
            pos = txt.rfind('\n', 0, pos-1)
            maxloop -= 1
            if maxloop < 0:
                break
        return segment

    def diff(self, inp, outp, file_name):
        import difflib
        inl = inp.splitlines(True)  # True for keepends
        outl = outp.splitlines(True)
        diff = difflib.unified_diff(inl, outl, file_name, 'round trip YAML')
        # 2.6 difflib has trailing space on filename lines %-)
        strip_trailing_space = sys.version_info < (2, 7)
        for line in diff:
            if strip_trailing_space and line[:4] in ['--- ', '+++ ']:
                line = line.rstrip() + '\n'
            sys.stdout.write(line)


def to_stdout(*args):
    sys.stdout.write(' '.join(args))


class YAML_Cmd(ProgramBase):
    def __init__(self):
        super(YAML_Cmd, self).__init__(
            formatter_class=SmartFormatter,
            aliases=True,
        )
        self._config = None

    # you can put these on __init__, but subclassing YAML_Cmd
    # will cause that to break
    @option('--verbose', '-v',
            help='increase verbosity level', action=CountAction,
            const=1, nargs=0, default=0, global_option=True)
    @option('--indent', type=int, global_option=True)
    @version('version: ' + __version__)
    def _pb_init(self):
        # special name for which attribs are included in help
        pass

    def run(self):
        self._yaml = YAML(self._args, self._config)
        if self._args.func:
            return self._args.func()

    def parse_args(self):
        # self._config = AppConfig(
        #     'yaml',
        #     filename=AppConfig.check,
        #     parser=self._parser,  # sets --config option
        #     warning=to_stdout,
        #     add_save=False,  # add a --save-defaults (to config) option
        # )
        # self._config._file_name can be handed to objects that need
        # to get other information from the configuration directory
        # self._config.set_defaults()
        self._parse_args()

    @sub_parser(
        aliases=['round-trip'],
        help='test round trip on YAML data',
        description='test round trip on YAML data',
    )
    @option('--save', action='store_true', help="""save the rewritten data back
    to the input file (if it doesn't exist a '.orig' backup will be made)
    """)
    @option('file', nargs='+')
    def rt(self):
        return self._yaml.round_trip()

    @sub_parser(
        aliases=['from-json'],
        help='convert json to block YAML',
        description='convert json to block YAML',
    )
    @option('--flow', action='store_true',
            help='use flow instead of block style')
    #@option('--literal', action='store_true',
    #        help='convert scalars with newlines to literal block style')
    @option('file', nargs='+')
    def json(self):
        return self._yaml.from_json()

    @sub_parser(
        aliases=['from-ini'],
        help='convert .ini/config to block YAML',
        description='convert .ini/config to block YAML',
    )
    @option('--basename', '-b', action='store_true',
            help='re-use basename of file for .yaml file, instead of writing'
            ' to stdout')
    @option('--test', action='store_true')
    @option('file')
    def ini(self):
        return self._yaml.from_ini()

    @sub_parser(
        #aliases=['to-html'],
        help='convert YAML to html tables',
        description="""convert YAML to html tables. If hierarchy is two deep (
        sequence/mapping over sequence/mapping) this is mapped to one table
        If the hierarchy is three deep, a list of 2 deep tables is assumed, but
        any non-list/mapp second level items are considered text.
        Row level keys are inserted in first column (unless --no-row-key),
        item level keys are used as classes for the TD.
        """,
    )
    @option("--level", action='store_true', help="print # levels and exit")
    @option('file')
    def htmltable(self):
        return self._yaml.to_htmltable()

    @sub_parser('from-html',
        help='convert HTML to YAML',
        description="""convert HTML to YAML. Tags become keys with as
        value a list. The first item in the list is a key value pair with
        key ".attribute" if attributes are available followed by tag and string
        segment items. Lists with one item are by default flattened.
        """,
    )
    @option("--no-body", action='store_true',
            help="drop top level html and body from HTML code segments")
    @option("--strip", action='store_true',
            help="strip whitespace surrounding strings")
    @option('file')
    def from_html(self):
        return self._yaml.from_html()

    @sub_parser('from-csv',
        aliases=['csv'],
        help='convert CSV to YAML',
        description="""convert CSV to YAML.
        By default generates a list of rows, with the items in a 2nd level
        list.
        """,
    )
    @option('--delimiter')
    #@option('--quotechar')
    @option('file')
    def from_csv(self):
        return self._yaml.from_csv()

    if 'test' in sys.argv:
        @sub_parser(
            description='internal test function',
        )
        @option('file', nargs='*')
        def test(self):
            return self._yaml.test()


def main():
    n = YAML_Cmd()
    n.parse_args()
    sys.exit(n.run())




from __future__ import print_function

"""
various test cases for string scalars in YAML files
'|' for preserved newlines
'>' for folded (newlines become spaces)

and the chomping modifiers:
'-' for stripping: final line break and any trailing empty lines are excluded
'+' for keeping: final line break and empty lines are preserved
''  for clipping: final line break preserved, empty lines at end not
    included in content (no modifier)

"""

import pytest

import ruamel.yaml
from ruamel.yaml.compat import ordereddict
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump


class TestYAML:
    def test_basic_string(self):
        round_trip("""
        a: abcdefg
        """, )

    def test_quoted_string(self):
        round_trip("""
        a: '12345'
        """)

    def test_preserve_string(self):
        round_trip("""
            a: |
              abc
              def
            """, intermediate=dict(a='abc\ndef\n'))

    def test_preserve_string_strip(self):
        s = """
            a: |-
              abc
              def

            """
        o = dedent(s).rstrip() + '\n'
        round_trip(s, outp=o, intermediate=dict(a='abc\ndef'))

    def test_preserve_string_keep(self):
            # with pytest.raises(AssertionError) as excinfo:
            round_trip("""
            a: |+
              ghi
              jkl


            b: x
            """, intermediate=dict(a='ghi\njkl\n\n\n', b='x'))

    def test_preserve_string_keep_at_end(self):
        # at EOF you have to specify the ... to get proper "closure"
        # of the multiline scalar
        round_trip("""
            a: |+
              ghi
              jkl

            ...
            """, intermediate=dict(a='ghi\njkl\n\n'))

    def test_fold_string(self):
        with pytest.raises(AssertionError) as excinfo:
            round_trip("""
            a: >
              abc
              def

            """, intermediate=dict(a='abc def\n'))

    def test_fold_string_strip(self):
        with pytest.raises(AssertionError) as excinfo:
            round_trip("""
            a: >-
              abc
              def

            """, intermediate=dict(a='abc def'))

    def test_fold_string_keep(self):
        with pytest.raises(AssertionError) as excinfo:
            round_trip("""
            a: >+
              abc
              def

            """, intermediate=dict(a='abc def\n\n'))




from __future__ import print_function

import io
import sys
import subprocess
try:
    _ = subprocess.check_output

    def check_output(*args, **kw):
        try:
            res = subprocess.check_output(*args, **kw)
        except subprocess.CalledProcessError as e:
            print("subprocess.CalledProcessError\n", e.output, sep='')
            res = e.output
        if PY3:
            res = res.decode('utf-8')
        return res
except AttributeError:
    def check_output(*args, **kw):
        process = subprocess.Popen(stdout=subprocess.PIPE, *args, **kw)
        output, unused_err = process.communicate()
        if PY3:
            output = output.decode('utf-8')
        return output

import pytest

import ruamel.yaml
from ruamel.yaml.compat import PY3
from roundtrip import dedent


def call_util(s, file_name, cmd, mp, td):
    """call the utilitiy yaml. if exit != 0 or if somethhing goes wrong
    return error output"""
    mp.chdir(td)
    with io.open(file_name, 'w') as fp:
        fp.write(dedent(s))
    res = check_output(cmd, stderr=subprocess.STDOUT)
    return res.replace('\r\n', '\n')


def rt_test(s, file_name, mp, td):
    return call_util(s, file_name, ['yaml', 'rt', "-v", file_name], mp, td)


class TestUtil:

    def test_version(self, capsys):
        res = check_output(
            ['yaml', '--version'], stderr=subprocess.STDOUT)
        assert res.replace('\r\n', '\n') == \
               u"version: {0}\n".format(ruamel.yaml.__version__)

    def test_ok(self, tmpdir, monkeypatch):
        file_name = "00_ok.yaml"
        res = rt_test(u"""
        - abc
        - ghi  # some comment
        - klm
        """, file_name, mp=monkeypatch, td=tmpdir)
        assert res == "{0}: ok\n".format(file_name)

    def test_not_ok(self, tmpdir, monkeypatch):
        file_name = "01_second_rt_ok.yaml"
        res = rt_test(u"""
        - abc
        -  ghi # some comment
        - klm
        """, file_name, mp=monkeypatch, td=tmpdir)
        assert res.replace('\r\n', '\n') == dedent("""
        {file_name}:
             stabilizes on second round trip, ok without comments
        --- 01_second_rt_ok.yaml
        +++ round trip YAML
        @@ -1,3 +1,3 @@
         - abc
        --  ghi # some comment
        +- ghi  # some comment
         - klm
        """).format(**dict(file_name=file_name))

    def test_from_configobj(self, tmpdir, monkeypatch):
        file_name = "02_from_ini.yaml"
        res = call_util(
            u"""
        # initial comment

        keyword1 = value1
        keyword2 = value2

        [section 1]
        keyword1 = value1
        keyword2 = value2

            [[sub-section]]
            # this is in section 1
            keyword1 = value1
            keyword2 = value2

                [[[nested section]]]
                # this is in sub section
                keyword1 = value1
                keyword2 = value2

            [[sub-section2]]
            # this is in section 1 again
            keyword1 = value1
            keyword2 = value2

        [[sub-section3]]
        # this is also in section 1, indentation is misleading here
        keyword1 = value1
        keyword2 = value2

        # final comment
        """, file_name, ['yaml', 'from-ini', file_name],
            mp=monkeypatch, td=tmpdir)
        print(res)
        assert res.replace('\r\n', '\n') == dedent("""
        # initial comment
        keyword1: value1
        keyword2: value2
        section 1:
          keyword1: value1
          keyword2: value2
          sub-section:
            # this is in section 1
            keyword1: value1
            keyword2: value2
            nested section:
              # this is in sub section
              keyword1: value1
              keyword2: value2
          sub-section2:
            # this is in section 1 again
            keyword1: value1
            keyword2: value2
          sub-section3:
            # this is also in section 1, indentation is misleading here
            keyword1: value1
            keyword2: value2
        # final comment
        """)

    def test_from_configobj_extra_comments(self, tmpdir, monkeypatch):
        file_name = "02_from_ini.yaml"
        res = call_util(
            u"""
        # initial comment
        keyword1 = value1
        keyword2 = value2  # eol comment kw2

        [section 1]
        keyword1 = value1 # and here more comment
        # comment s1kw2
        keyword2 = value2  # eol comment s1kw2

            [[sub-section]]  # eol on section
            # this is in section 1
            keyword1 = value1
            keyword2 = value2

                [[[nested section]]]
                # this is in sub section
                keyword1 = value1
                keyword2 = value2
                # after nested

            [[sub-section2]]
            # this is in section 1 again
            keyword1 = value1
            keyword2 = value2

        [[sub-section3]] # comment on section key
        # this is also in section 1, indentation is misleading here
        keyword1 = value1
        keyword2 = value2

        # final comment
        """,
            file_name, ['yaml', 'from-ini', file_name],
            mp=monkeypatch, td=tmpdir)
        print(res)
        assert res.replace('\r\n', '\n') == dedent("""
        # initial comment
        keyword1: value1
        keyword2: value2 # eol comment kw2
        section 1:
          keyword1: value1 # and here more comment
          # comment s1kw2
          keyword2: value2 # eol comment s1kw2
          sub-section: # eol on section
            # this is in section 1
            keyword1: value1
            keyword2: value2
            nested section:
              # this is in sub section
              keyword1: value1
              keyword2: value2
          # after nested
          sub-section2:
            # this is in section 1 again
            keyword1: value1
            keyword2: value2
          sub-section3: # comment on section key
            # this is also in section 1, indentation is misleading here
            keyword1: value1
            keyword2: value2
        # final comment
        """)



# coding: utf-8

from __future__ import print_function

import pytest
from textwrap import dedent

import ruamel.yaml
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump


def load(s):
    return round_trip_load(dedent(s))


def compare(data, s):
    assert round_trip_dump(data) == dedent(s)
#@pytest.mark.xfail


class TestCommentsManipulation:

    # list
    def test_seq_set_comment_on_existing_explicit_column(self):
        data = load("""
        - a   # comment 1
        - b
        - c
        """)
        data.yaml_add_eol_comment('comment 2', key=1, column=6)
        compare(data, """
        - a   # comment 1
        - b   # comment 2
        - c
        """)

    def test_seq_overwrite_comment_on_existing_explicit_column(self):
        data = load("""
        - a   # comment 1
        - b
        - c
        """)
        data.yaml_add_eol_comment('comment 2', key=0, column=6)
        compare(data, """
        - a   # comment 2
        - b
        - c
        """)

    def test_seq_first_comment_explicit_column(self):
        data = load("""
        - a
        - b
        - c
        """)
        data.yaml_add_eol_comment('comment 1', key=1, column=6)
        compare(data, """
        - a
        - b   # comment 1
        - c
        """)

    def test_seq_set_comment_on_existing_column_prev(self):
        data = load("""
        - a   # comment 1
        - b
        - c
        - d     # comment 3
        """)
        data.yaml_add_eol_comment('comment 2', key=1)
        compare(data, """
        - a   # comment 1
        - b   # comment 2
        - c
        - d     # comment 3
        """)

    def test_seq_set_comment_on_existing_column_next(self):
        data = load("""
        - a   # comment 1
        - b
        - c
        - d     # comment 3
        """)
        print(data._yaml_comment)
        # print(type(data._yaml_comment._items[0][0].start_mark))
        # ruamel.yaml.error.Mark
        #print(type(data._yaml_comment._items[0][0].start_mark))
        data.yaml_add_eol_comment('comment 2', key=2)
        compare(data, """
        - a   # comment 1
        - b
        - c     # comment 2
        - d     # comment 3
        """)

    def test_seq_set_comment_on_existing_column_further_away(self):
        """
        no comment line before or after, take the latest before
        the new position
        """
        data = load("""
        - a   # comment 1
        - b
        - c
        - d
        - e
        - f     # comment 3
        """)
        print(data._yaml_comment)
        # print(type(data._yaml_comment._items[0][0].start_mark))
        # ruamel.yaml.error.Mark
        #print(type(data._yaml_comment._items[0][0].start_mark))
        data.yaml_add_eol_comment('comment 2', key=3)
        compare(data, """
        - a   # comment 1
        - b
        - c
        - d   # comment 2
        - e
        - f     # comment 3
        """)

    def test_seq_set_comment_on_existing_explicit_column_with_hash(self):
        data = load("""
        - a   # comment 1
        - b
        - c
        """)
        data.yaml_add_eol_comment('#  comment 2', key=1, column=6)
        compare(data, """
        - a   # comment 1
        - b   #  comment 2
        - c
        """)

    # dict

    def test_dict_set_comment_on_existing_explicit_column(self):
        data = load("""
        a: 1   # comment 1
        b: 2
        c: 3
        d: 4
        e: 5
        """)
        data.yaml_add_eol_comment('comment 2', key='c', column=7)
        compare(data, """
        a: 1   # comment 1
        b: 2
        c: 3   # comment 2
        d: 4
        e: 5
        """)

    def test_dict_overwrite_comment_on_existing_explicit_column(self):
        data = load("""
        a: 1   # comment 1
        b: 2
        c: 3
        d: 4
        e: 5
        """)
        data.yaml_add_eol_comment('comment 2', key='a', column=7)
        compare(data, """
        a: 1   # comment 2
        b: 2
        c: 3
        d: 4
        e: 5
        """)


    def test_map_set_comment_on_existing_column_prev(self):
        data = load("""
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """)
        data.yaml_add_eol_comment('comment 2', key='b')
        compare(data, """
            a: 1   # comment 1
            b: 2   # comment 2
            c: 3
            d: 4
            e: 5     # comment 3
            """)

    def test_map_set_comment_on_existing_column_next(self):
        data = load("""
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """)
        data.yaml_add_eol_comment('comment 2', key='d')
        compare(data, """
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4     # comment 2
            e: 5     # comment 3
            """)

    def test_map_set_comment_on_existing_column_further_away(self):
        """
        no comment line before or after, take the latest before
        the new position
        """
        data = load("""
            a: 1   # comment 1
            b: 2
            c: 3
            d: 4
            e: 5     # comment 3
            """)
        data.yaml_add_eol_comment('comment 2', key='c')
        print(round_trip_dump(data))
        compare(data, """
            a: 1   # comment 1
            b: 2
            c: 3   # comment 2
            d: 4
            e: 5     # comment 3
            """)

# the ugly {comment} in the following is because
# py.test cannot handle comments in strings properly
# https://bitbucket.org/pytest-dev/pytest/issue/752/internalerror-indexerror-list-index-out-of

    #@pytest.mark.xfail
    def test_before_top_map_rt(self):
        data = load("""
        a: 1
        b: 2
        """)
        data.yaml_set_start_comment('Hello\nWorld\n')
        compare(data, """
        {comment} Hello
        {comment} World
        a: 1
        b: 2
        """.format(comment='#'))

    def test_before_top_map_replace(self):
        data = load("""
        # abc
        # def
        a: 1 # 1
        b: 2
        """)
        data.yaml_set_start_comment('Hello\nWorld\n')
        compare(data, """
        {comment} Hello
        {comment} World
        a: 1 # 1
        b: 2
        """.format(comment='#'))

    def test_before_top_map_from_scratch(self):
        from ruamel.yaml.comments import CommentedMap
        data = CommentedMap()
        data['a'] = 1
        data['b'] = 2
        data.yaml_set_start_comment('Hello\nWorld\n')
        #print(data.ca)
        #print(data.ca._items)
        compare(data, """
            {comment} Hello
            {comment} World
            a: 1
            b: 2
            """.format(comment='#'))

    def test_before_top_seq_rt(self):
        data = load("""
        - a
        - b
        """)
        data.yaml_set_start_comment('Hello\nWorld\n')
        print(round_trip_dump(data))
        compare(data, """
        # Hello
        # World
        - a
        - b
        """)

    def test_before_top_seq_rt_replace(self):
        data = load("""
        {comment} this
        {comment} that
        - a
        - b
        """.format(comment='#'))
        data.yaml_set_start_comment('Hello\nWorld\n')
        print(round_trip_dump(data))
        compare(data, """
        {comment} Hello
        {comment} World
        - a
        - b
        """.format(comment='#'))

    def test_before_top_seq_from_scratch(self):
        from ruamel.yaml.comments import CommentedSeq
        data = CommentedSeq()
        data.append('a')
        data.append('b')
        data.yaml_set_start_comment('Hello\nWorld\n')
        print(round_trip_dump(data))
        compare(data, """
        {comment} Hello
        {comment} World
        - a
        - b
        """.format(comment='#'))

    # nested variants
    def test_before_nested_map_rt(self):
        data = load("""
        a: 1
        b:
          c: 2
          d: 3
        """)
        data['b'].yaml_set_start_comment('Hello\nWorld\n')
        compare(data, """
        a: 1
        b:
        {comment} Hello
        {comment} World
          c: 2
          d: 3
        """.format(comment='#'))

    def test_before_nested_map_rt_indent(self):
        data = load("""
        a: 1
        b:
          c: 2
          d: 3
        """)
        data['b'].yaml_set_start_comment('Hello\nWorld\n', indent=2)
        compare(data, """
        a: 1
        b:
          {comment} Hello
          {comment} World
          c: 2
          d: 3
        """.format(comment='#'))
        print(data['b'].ca)

    def test_before_nested_map_from_scratch(self):
        from ruamel.yaml.comments import CommentedMap
        data = CommentedMap()
        datab = CommentedMap()
        data['a'] = 1
        data['b'] = datab
        datab['c'] = 2
        datab['d'] = 3
        data['b'].yaml_set_start_comment('Hello\nWorld\n')
        compare(data, """
        a: 1
        b:
        {comment} Hello
        {comment} World
          c: 2
          d: 3
        """.format(comment='#'))

    def test_before_nested_seq_from_scratch(self):
        from ruamel.yaml.comments import CommentedMap, CommentedSeq
        data = CommentedMap()
        datab = CommentedSeq()
        data['a'] = 1
        data['b'] = datab
        datab.append('c')
        datab.append('d')
        data['b'].yaml_set_start_comment('Hello\nWorld\n', indent=2)
        compare(data, """
        a: 1
        b:
          {comment} Hello
          {comment} World
          - c
          - d
        """.format(comment='#'))




#! /usr/bin/env python
# coding: utf-8

from __future__ import print_function

import sys
import os
import platform
from textwrap import dedent

name_space = 'ruamel'
package_name = 'yaml'
full_package_name = name_space + '.' + package_name

exclude_files = [
    'setup.py',
]


def get_version():
    v_i = 'version_info = '
    for line in open('py/__init__.py'):
        if not line.startswith(v_i):
            continue
        s_e = line[len(v_i):].strip()[1:-1].split(', ')
        els = [x.strip()[1:-1] if x[0] in '\'"' else int(x) for x in s_e]
        return els


def _check_convert_version(tup):
    """create a PEP 386 pseudo-format conformant string from tuple tup"""
    ret_val = str(tup[0])  # first is always digit
    next_sep = "."  # separator for next extension, can be "" or "."
    nr_digits = 0  # nr of adjacent digits in rest, to verify
    post_dev = False  # are we processig post/dev
    for x in tup[1:]:
        if isinstance(x, int):
            nr_digits += 1
            if nr_digits > 2:
                raise ValueError("too many consecutive digits " + ret_val)
            ret_val += next_sep + str(x)
            next_sep = '.'
            continue
        first_letter = x[0].lower()
        next_sep = ''
        if first_letter in 'abcr':
            if post_dev:
                raise ValueError("release level specified after "
                                 "post/dev:" + x)
            nr_digits = 0
            ret_val += 'rc' if first_letter == 'r' else first_letter
        elif first_letter in 'pd':
            nr_digits = 1  # only one can follow
            post_dev = True
            ret_val += '.post' if first_letter == 'p' else '.dev'
        else:
            raise ValueError('First letter of "' + x + '" not recognised')
    return ret_val


version_info = get_version()
version_str = _check_convert_version(version_info)

if __name__ == '__main__':
    # put here so setup.py can be imported more easily
    from setuptools import setup, find_packages, Extension, Distribution
    from setuptools.command import install_lib

    # windows things this is pure, that way you would get pure python
    # whl files that take precedence on Linux over source with compilable C
    if '--universal' in sys.argv:
        Distribution.is_pure = lambda *args: True
    Distribution.is_pure = lambda *args: False



class MyInstallLib(install_lib.install_lib):
    "create __init__.py on the fly"
    def run(self):
        install_lib.install_lib.run(self)

    def install(self):
        fpp = full_package_name.split('.')  # full package path
        full_exclude_files = [os.path.join(*(fpp + [x]))
                              for x in exclude_files]
        alt_files = []
        outfiles = install_lib.install_lib.install(self)
        for x in outfiles:
            for full_exclude_file in full_exclude_files:
                if full_exclude_file in x:
                    os.remove(x)
                    break
            else:
                alt_files.append(x)
        return alt_files


def check_extensions():
    """check if the C module can be build by trying to compile a small
    program against the libyaml development library"""
    if sys.platform == "win32":
        return None
    import tempfile
    import shutil

    import distutils.sysconfig
    import distutils.ccompiler
    from distutils.errors import CompileError, LinkError

    libraries = ['yaml']

    # write a temporary .c file to compile
    c_code = dedent("""
    #include <yaml.h>

    int main(int argc, char* argv[])
    {
        yaml_parser_t parser;
        parser = parser;  /* prevent warning */
        return 0;
    }
    """)
    tmp_dir = tempfile.mkdtemp(prefix='tmp_ruamel_yaml_')
    ret_val = None
    try:
        bin_file_name = os.path.join(tmp_dir, 'test_yaml')
        file_name = bin_file_name + '.c'
        with open(file_name, 'w') as fp:
            fp.write(c_code)

        # and try to compile it
        compiler = distutils.ccompiler.new_compiler()
        assert isinstance(compiler, distutils.ccompiler.CCompiler)
        distutils.sysconfig.customize_compiler(compiler)

        try:
            compiler.link_executable(
                compiler.compile([file_name]),
                bin_file_name,
                libraries=libraries,
            )
        except CompileError:
            print('libyaml compile error')
        except LinkError:
            print('libyaml link error')
        else:
            ret_val = [
                Extension(
                    '_yaml',
                    sources=['ext/_yaml.c'],
                    libraries=libraries,
                    ),
            ]
    except:
        pass
    finally:
        shutil.rmtree(tmp_dir)
    return ret_val


def main():
    install_requires = [
        "ruamel.base",
        "ruamel.std.argparse>=0.5.1",
    ]
    # use fast ordereddict for !!omap
    if sys.version_info[0] == 2 and \
       platform.python_implementation() == "CPython":
        install_requires.extend(['ruamel.ordereddict'])
    # if sys.version_info < (3, 4):
    #     install_requires.append("")
    packages = [full_package_name] + [
        (full_package_name + '.' + x)
        for x in find_packages('py', exclude=['test'])]
    setup(
        name=full_package_name,
        version=version_str,
        description=full_package_name + " is a YAML parser/emitter that "
        "supports roundtrip preservation of comments, seq/map flow style, and "
        "map key order",
        install_requires=install_requires,
        long_description=open('README.rst').read(),
        url='https://bitbucket.org/ruamel/' + package_name,
        author='Anthon van der Neut',
        author_email='a.van.der.neut@ruamel.eu',
        license="MIT license",
        package_dir={full_package_name: 'py'},
        namespace_packages=[name_space],
        packages=packages,
        ext_modules=check_extensions(),
        entry_points=mk_entry_points(full_package_name),
        cmdclass={'install_lib': MyInstallLib},
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            "Programming Language :: Python :: 2.6",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.3",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Topic :: Text Processing :: Markup",
        ]
    )


def mk_entry_points(full_package_name):
    script_name = full_package_name.rsplit('.', 1)[-1]
    return {'console_scripts': [
        '{0} = {1}:main'.format(script_name, full_package_name),
    ]}

if __name__ == '__main__':
    if 'yamlctest' in sys.argv:
        print(check_extensions())
        sys.exit(1)
    main()



# coding: utf-8

from __future__ import print_function

import dateutil.parser
import ruamel.yaml

class CSV2YAML(object):
    def __init__(self, args=None):
        # self.flatten = not getattr(args, 'no_flatten', False)
        self.delimiter = getattr(args, 'delimiter', None)

    def __call__(self, csv_file_name):
        import csv

        data = []
        with open(csv_file_name) as inf:
            for line in csv.reader(inf, delimiter=self.delimiter):
                data.append(self.process_line(line))
        print(ruamel.yaml.dump(data, Dumper=ruamel.yaml.RoundTripDumper))

    def process_line(self, line):
        """convert lines, trying, int, float, date"""
        ret_val = []
        for elem in line:
            try:
                res = int(elem)
                ret_val.append(res)
                continue
            except ValueError:
                pass
            try:
                res = float(elem)
                ret_val.append(res)
                continue
            except ValueError:
                pass
            try:
                res = dateutil.parser.parse(elem)
                ret_val.append(res)
                continue
            except TypeError:
                pass
            ret_val.append(elem)
        return ret_val






def configobj_walker(cfg):
    """
    walks over a ConfigObj (INI file with comments) generating
    corresponding YAML output (including comments
    """
    from configobj import ConfigObj
    assert isinstance(cfg, ConfigObj)
    for c in cfg.initial_comment:
        if c.strip():
            yield c
    for s in _walk_section(cfg):
        if s.strip():
            yield s
    for c in cfg.final_comment:
        if c.strip():
            yield c


def _walk_section(s, level=0):
    from configobj import Section
    assert isinstance(s, Section)
    indent = '  ' * level
    for name in s.scalars:
        for c in s.comments[name]:
            yield indent + c.strip()
        line = '{0}{1}: {2}'.format(indent, name, s[name])
        c = s.inline_comments[name]
        if c:
            line += ' ' + c
        yield line
    for name in s.sections:
        for c in s.comments[name]:
            yield indent + c.strip()
        line = '{0}{1}:'.format(indent, name)
        c = s.inline_comments[name]
        if c:
            line += ' ' + c
        yield line
        for val in _walk_section(s[name], level=level+1):
            yield val




from __future__ import print_function

import sys
import os
import pytest

sys.path.insert(0, os.path.dirname(__file__) + '/lib')

import ruamel.yaml
import test_appliance

args = []


def test_data():
    collections = []
    import test_yaml
    collections.append(test_yaml)
    test_appliance.run(collections, args)


# @pytest.mark.skipif(not ruamel.yaml.__with_libyaml__,
#                     reason="no libyaml")
def test_data_ext():
    collections = []
    if ruamel.yaml.__with_libyaml__:
        import test_yaml_ext
        collections.append(test_yaml_ext)
        test_appliance.run(collections, args)
